#ifndef __rbtree_h_
#define __rbtree_h_

//*************************************************************************
// rbtree.h - code for a simple red-black tree.
//
// rbtreeType<X> is a red-black tree with elements of type X.  If X is
// a simple type, like int, then rbtreeType<X> forms a set with elements
// of type <X>.  If X is any ordered pair type, and only the first item
// in the pair is used in the comparison, then rbtreeType forms a lookup
// table.  All operations are O(log(n)), where n is the number of elements
// in the three, except as noted below.
//
// The type X must have a < and == operator.  If two items are == then only
// one of them may be in the tree at once.  If A < B, A will come before A
// when iterating with the next() function.
//
// The assumption is that < and == form a consistent ordering on type X.
// However, even if these operators give bogus results, the code will
// never dump core or run in an infinite loop.  In fact, the algorithms
// will still be O(log(n)), even in the worst case.  However, the size
// of the tree may be larger if the == operation always returns false.
// Iterating through the tree will still return all elements of the tree.
//
// There is no current way for < and == to fail.  Future plans will allow
// those operators to safely throw exceptions.
//
// The copies of the items, not pointers to the items, are stored in the
// tree.  If X is a class, is needs to have a valid copy constructor,
// assignment operator, and destructor.  References are used as much as
// possible to minimize the number of copies.
//
// The tree defines an iterator type.  The iterator is a constant
// pointer to a node.  A constant reference to the data is available 
// from here.  Overriding the const is perfectly acceptable as long as
// the new value == the original value.  (Note that the const on the
// iterator is more a consequence of some nasty implementation details
// than it is a suggestion of how to use the iterator.)  A NULL iterator
// indicates the end of the tree, or a non-existent element.  All 
// iterators are valid only as long as the tree does not changes; any
// additions or deletions invalidates all iterators for the given tree.
//
// Define UNIT_TEST to run tests on a simple instantiation of this
// datatype.  Also, the UNIT_TEST code provides a simple sample of how
// to use this tree.  Define RBTREE_WITH_DEBUG to compile methods which
// are used in development and debugging, but are not needed in a working
// system.  RBTREE_WITH_DEBUG requires console output, so the code is
// not applicable for all systems.
//
// The algorithms below were adapted from _Introduction_to_Algorithms_
// by Cormen, Leiserson, and Rivest, 1990.  This book is an excellent
// source for information on this data structure.
//
// All code below was written by Philip Smolen, copyright 2000, and
// is released free under the GNU Public License.  (This covers the
// code, not the algorithms.)  Contact info:  psmole19@idt.net.

#ifdef UNIT_TEST
#define RBTREE_WITH_DEBUG
#endif

#include <assert.h>
#include <stdlib.h>

#ifdef RBTREE_WITH_DEBUG
#include <iostream>
#endif


template <class dataType>
class rbtreeType
{
public:
  struct nodeType
  { // See any reference on red-black trees for more info on this structure.
    friend rbtreeType;
  private:
    dataType data;
    nodeType *left, *right, *parent;
    bool black;
    nodeType(dataType const &initialData) : data(initialData) {}
    nodeType() {}
  public:
    // Access the data.
    const dataType &getData() const {return data;}
  };

  // Exporting an iterator, rather than directly returning the data,
  // makes certain operations fasters.
  typedef const nodeType *iteratorType;


  // Create an empty tree.
  rbtreeType();


  // The following four methods create a set datatype.  These methods,
  // alone, will produce a datastructure equivilant to a hash table,
  // but slightly less efficiently.

  // add() inserts a new data item into the set, returning true on success.
  // If the item does not exists in the set, the operation always succeeds.
  // If the item already exists in the set, and allowOverwrite is true, the
  // old item will be replaced by the new item, and the add will succeed.
  // If the item already exists in the set, the add will fail and the tree
  // will be left unchanged.
  bool add(dataType data, bool allowOverwrite = true);	   

  // remove() looks for the given item in the tree.  If it exists, it is
  // removed and the function returns true.  If it does not exists, the
  // tree is not changed and the function returns false.
  bool remove(dataType data);

  // exists() is the same as remove, except that it never changes the tree.
  bool exists(dataType data) const;

  // clear() removes all items from the tree.  It runs in O(n) time and
  // is much more effecient than removing each element one at a time.
  void clear();


  // The next set of methods allows iteration through the tree based on
  // the results of the < operator.  These make the tree more powerful
  // than a hash table.

  // These return iterators to the first/last elements of the tree.
  // They return NULL if the tree is empty.
  iteratorType first() const;
  iteratorType last() const;

  // find() looks for the given tiem in the tree.  If it finds the item,
  // it returns an intertor pointing to it.  If it does not find the item,
  // and the direction is exact, it returns NULL.  If it does not find
  // the item, and the direction is not exact, it returns the item that
  // would be immediately before/after the given item, if the given
  // item existed, or NULL if the given item would be the first/last in
  // the tree. 
  enum directionType {before, exact, after};
  iteratorType find(const dataType &data, directionType direction) const;

  // Given a valid iterator, these return the iterator for the next/previous,
  // item in the tree.  They return NULL if the the iterator points to the
  // last/first item in the tree.  next(NULL) is the same as first() and
  // prev(NULL) is the same as last().
  iteratorType next(iteratorType node) const;
  iteratorType prev(iteratorType node) const;


  // These provide standard memory management.  Copying the tree always makes
  // a deep copy, copying all the internal nodes.  This not very fast,
  // O(n log(n)).  The target app already does reference counting and copy
  // on write, so we don't need to repeat that.  We could, however, make
  // the copy more effecient, bringing it to O(n).
  rbtreeType(const rbtreeType &another);
  rbtreeType &operator = (const rbtreeType &another);
  ~rbtreeType() { clear(); }

#ifdef RBTREE_WITH_DEBUG
  // These provide simple debugging features.  dump() sends the tree to the
  // standard output.  assertInvarients tests the integerity of the entire
  // tree.  If it passes, it returns silently.  If it fails, it sends
  // information about what failed to the standard error, then terminates
  // the program.  Both take a significant amount of time, and are not meant
  // for use outside of a debugging environment.  Both assume X knows how
  // to stream itself with <<.
  void dump();
  void assertInvariants();
#endif

private:
  bool isNull(nodeType const *node) const {return node == &null;}
  iteratorType export(iteratorType node) const {return isNull(node)?NULL:node;}
  nodeType *newNode(dataType const &data, bool black);
  void remove(nodeType *node);

  void leftRotate(nodeType *node);
  void rightRotate(nodeType *node);

  nodeType *deepCopyNode(nodeType *parent,
			 const nodeType *otherNode, 
			 const rbtreeType &otherTree);
  void clearNode(nodeType *node);

#ifdef RBTREE_WITH_DEBUG
  void dumpNode(nodeType *node);
  void assertNodeInvariants(nodeType *toCheck, nodeType *parent, 
			    int &blackHeight, bool &problemFound);
#endif

  nodeType null;
  nodeType *root;
};


template <class dataType>
void rbtreeType<dataType>::clear() 
{
  clearNode(root);
}

template <class dataType>
void rbtreeType<dataType>::clearNode(nodeType *node)
{
  if (!isNull(node))
    {
      clearNode(node->left);
      clearNode(node->right);
      delete(node);
    }
}

template <class dataType>
rbtreeType<dataType>::rbtreeType() :
  root(&null)
{
  null.black = true;
  null.left = null.right = null.parent = &null;
}

template <class dataType>
rbtreeType<dataType>::rbtreeType(const rbtreeType &another)
{
  null.black = true;
  null.left = null.right = null.parent = &null;
  root = deepCopyNode(&null, another.root, another);
}

// room to speed up.
template <class dataType>
rbtreeType<dataType>::nodeType *rbtreeType<dataType>::deepCopyNode
(nodeType *parent, const nodeType *otherNode, const rbtreeType &otherTree)
{
  if (otherTree.isNull(otherNode))
    {
      return &null;
    }
  
  nodeType *node = newNode(otherNode->data, otherNode->black);
  node->parent = parent;
  node->left = deepCopyNode(node, otherNode->left, otherTree);
  node->right = deepCopyNode(node, otherNode->right, otherTree);
  return node;
}

#ifdef RBTREE_WITH_DEBUG
template <class dataType>
void rbtreeType<dataType>::assertInvariants()
{
  bool error = false;
  if (!root->black)
    {
      cerr<<"!root->black\n";
      error = true;
    }
  if (!null.black)
    {
      cerr<<"!null.black\n";
      error = true;
    }
  int blackHeight;
  bool otherProblem;
  assertNodeInvariants(root, &null, blackHeight, otherProblem);
  error = error || otherProblem;
  if (error)
    {
      dump();
      assert(false);
    }
}

template <class dataType>
void rbtreeType<dataType>::assertNodeInvariants(nodeType *toCheck, 
						nodeType *parent, 
						int &blackHeight, 
						bool &problemFound)
{
  blackHeight = toCheck->black?1:0;
  problemFound = false;

  if (isNull(toCheck))
    {
      return;
    }

  if (toCheck->parent != parent)
    {
      cerr<<"Invalid link to parent in node "<<toCheck->data<<'\n';
      problemFound = true;
    }

  int bhL, bhR;
  bool pfL, pfR;
  assertNodeInvariants(toCheck->left, toCheck, bhL, pfL);
  assertNodeInvariants(toCheck->right, toCheck, bhR, pfR);
  
  problemFound = problemFound || pfL || pfR;

  if ((bhL == -1) || (bhR == -1))
    {
      blackHeight = -1;
    }
  else if (bhL != bhR) 
    {
      cerr<<"Tree unbalanced at node "<<toCheck->data<<", left = "<<bhL
	  <<", right = "<<bhR<<'\n';
      blackHeight = -1;
      problemFound = true;
    }
  else 
    {
      blackHeight += bhL;
    }
}

template <class dataType>
void rbtreeType<dataType>::dump()
{
  dumpNode(root);
}

template <class dataType>
void rbtreeType<dataType>::dumpNode(nodeType *node)
{
  if (!isNull(node)) 
    {
      cout<<'(';
      dumpNode(node->left);
      cout<<' '<<node->data<<(node->black?'B':'r')<<' ';
      dumpNode(node->right);
      cout<<')';
    }
}
#endif

template <class dataType>
bool rbtreeType<dataType>::add(dataType data, bool allowOverwrite)
{
  if (isNull(root))
    {
      root = newNode(data, true);
      return true;
    }
      
  nodeType *parent = &null;
  nodeType *current = root;
  while (!isNull(current))
    {
      if (data == current->data)
	{ // The data is already in the tree.  (Or different data with the
	  // same key value.)
	  if (allowOverwrite) 
	    {
	      current->data = data;
	      return true;
	    }
	  else
	    {
	      return false;
	    }
	}

      parent = current;
      if (data < current->data) 
	{
	  current = current->left;
	}
      else
	{
	  current = current->right;
	}
    }

  // The data is not in the tree.  Parent points to a node with a null
  // pointer where we need to be inserted.
  current = newNode(data, false);
  current->parent = parent;
  if (data < parent->data) 
    {
      assert(isNull(parent->left));
      parent->left = current;
    }
  else
    {
      assert(isNull(parent->right));
      parent->right = current;
    }
  
  // We have added the new node to the binary search tree, but it may not
  // be a proper rbtree any more.  Now we need to force the tree back into
  // balance.
  while ((current != root) && (!current->parent->black)) 
    {
      assert (!isNull(current->parent->parent));  // Should be true because the
                                                 // root is always black.
      if (current->parent == current->parent->parent->left)
	{
	  nodeType *uncle = current->parent->parent->right;
	  if (!uncle->black)
	    {
	      current->parent->black = true;
	      uncle->black = true;
	      current->parent->parent->black = false;
	      current = current->parent->parent;
	    }
	  else 
	    {
	      if (current == current->parent->right) 
		{
		  current = current->parent;
		  leftRotate(current);
		}
	      current->parent->black = true;
	      current->parent->parent->black = false;
	      rightRotate(current->parent->parent);
	    }
	}
      else
	{
	  nodeType *uncle = current->parent->parent->left;
	  if (!uncle->black)
	    {
	      current->parent->black = true;
	      uncle->black = true;
	      current->parent->parent->black = false;
	      current = current->parent->parent;
	    }
	  else
	    {
	      if (current == current->parent->left) 
		{
		  current = current->parent;
		  rightRotate(current);
		}
	      current->parent->black = true;
	      current->parent->parent->black = false;
	      leftRotate(current->parent->parent);
	    }
	}
    }
  root->black = true;
  return true;
}

template <class dataType>
bool rbtreeType<dataType>::remove(dataType data)
{
  iteratorType parent = &null;
  nodeType *node = root;
  while (true) 
    {
      if (isNull(node)) 
	{ // The data we want to delete is not in the tree.
	  return false;
	}
      if (node->data == data)
	{ // This is the node we want to delete.
	  remove(node);
	  return true;
	}
      parent = node;
      if (data < node->data) 
	{
	  node = node->left;
	}
      else
	{
	  node = node->right;
	}
    }
}

template <class dataType>
void rbtreeType<dataType>::remove(nodeType *node)
{
  if (!node) { return; }

  const nodeType *reallyRemove;
  if (isNull(node->left) || isNull(node->right))
    {
      reallyRemove = node;
    }
  else
    {
      reallyRemove = next(node);
    }

  nodeType *soleChild;  // This may become the sentinal null, if there are
                        // no children.
  if (isNull(reallyRemove->left))
    {
      soleChild = reallyRemove->right;
    }
  else
    {
      soleChild = reallyRemove->left;
    }

  soleChild->parent = reallyRemove->parent;
  if (isNull(reallyRemove->parent))
    {
      root = soleChild;
    }
  else if (reallyRemove->parent->left == reallyRemove) 
    {
      reallyRemove->parent->left = soleChild;
    }
  else
    {
      reallyRemove->parent->right = soleChild;
    }
  if (reallyRemove != node)
    {
      node->data = reallyRemove->data;
    }

  // The data in the specified node has been removed from the tree.
  // The node reallyRemove has been removed from the tree.  Now the
  // tree may be out of balance, and not a valid red-black tree.
  if (reallyRemove->black)
    {
      nodeType *extraBlack = soleChild;
      while ((extraBlack != root) && extraBlack->black)
	{
	  if (extraBlack == extraBlack->parent->left)
	    {
	      nodeType *sibling = extraBlack->parent->right;
	      if (!sibling->black)
		{
		  sibling->black = true;
		  extraBlack->parent->black = false;
		  leftRotate(extraBlack->parent);
		  sibling = extraBlack->parent->right;
		}
	      if (sibling->left->black && sibling->right->black)
		{
		  sibling->black = false;
		  extraBlack = extraBlack->parent;
		}
	      else
		{
		  if (sibling->right->black)
		    {
		      sibling->left->black = true;
		      sibling->black = false;
		      rightRotate(sibling);
		      sibling = extraBlack->parent->right;
		    }
		  sibling->black = extraBlack->parent->black;
		  extraBlack->parent->black = true;
		  sibling->right->black = true;
		  leftRotate(extraBlack->parent);
		  extraBlack = root;
		}
	    }
	  else
	    {
	      nodeType *sibling = extraBlack->parent->left;
	      if (!sibling->black)
		{
		  sibling->black = true;
		  extraBlack->parent->black = false;
		  rightRotate(extraBlack->parent);
		  sibling = extraBlack->parent->left;
		}
	      if (sibling->left->black && sibling->right->black)
		{
		  sibling->black = false;
		  extraBlack = extraBlack->parent;
		}
	      else
		{
		  if (sibling->left->black)
		    {
		      sibling->right->black = true;
		      sibling->black = false;
		      leftRotate(sibling);
		      sibling = extraBlack->parent->left;
		    }
		  sibling->black = extraBlack->parent->black;
		  extraBlack->parent->black = true;
		  sibling->left->black = true;
		  rightRotate(extraBlack->parent);
		  extraBlack = root;
		}
	    }
	}
      extraBlack->black = true;
    }

  // Now that we are done, we can free up the node.
  delete(reallyRemove);
}

template <class dataType>
bool rbtreeType<dataType>::exists(dataType data) const
{
  nodeType *node = root;
  while (!isNull(node)) 
    {
      if (node->data == data)
	{
	  return true;
	}
      if (data < node->data) 
	{
	  node = node->left;
	}
      else
	{
	  node = node->right;
	}
    }
  return false;
}

template <class dataType>
rbtreeType<dataType>::nodeType *rbtreeType<dataType>::newNode(dataType const &data, 
							      bool black)
{
  nodeType *node = new nodeType(data);
  node->black = black;
  node->left = node->right = node->parent = &null;
  return node;
}

template <class dataType>
void rbtreeType<dataType>::leftRotate(nodeType *node)
{
  //cout<<'[';dump();cout<<" -> ";
  assert(!(isNull(node)||isNull(node->right)));
  nodeType *x = node;
  nodeType *y = x->right;
  x->right = y->left;
  if (!isNull(y->left))
    {
      y->left->parent = x;
    }
  y->parent = x->parent;
  if (isNull(x->parent))
    {
      root = y;
    }
  else
    {
      if (x == x->parent->left)
	{
	  x->parent->left = y;
	}
      else
	{
	  x->parent->right = y;
	}
    }
  y->left = x;
  x->parent = y;
  //dump();cout<<']';
}

template <class dataType>
void rbtreeType<dataType>::rightRotate(nodeType *node)
{
  assert(!(isNull(node)||isNull(node->left)));
  nodeType *x = node;
  nodeType *y = x->left;
  x->left = y->right;
  if (!isNull(y->right))
    {
      y->right->parent = x;
    }
  y->parent = x->parent;
  if (isNull(x->parent))
    {
      root = y;
    }
  else
    {
      if (x == x->parent->right)
	{
	  x->parent->right = y;
	}
      else
	{
	  x->parent->left = y;
	}
    }
  y->right = x;
  x->parent = y;
}

template <class dataType>
rbtreeType<dataType>::iteratorType rbtreeType<dataType>::first() const
{
  const nodeType *parent = &null;
  nodeType *current = root;
  while (!isNull(current))
    {
      parent = current;
      current = current->left;
    }
  return export(parent);
}  

template <class dataType>
rbtreeType<dataType>::iteratorType rbtreeType<dataType>::last() const
{
  iteratorType parent = &null;
  nodeType *current = root;
  while (!isNull(current))
    {
      parent = current;
      current = current->right;
    }
  return export(parent);
}

template <class dataType>
rbtreeType<dataType>::iteratorType 
rbtreeType<dataType>::find(const dataType &data, directionType direction) const
{
  if (isNull(root))
    { // This will have to be a special case sooner or later.  Might as
      // well deal with it now.
      return NULL;
    }

  iteratorType parent = &null;
  nodeType *node = root;
  while (!isNull(node)) 
    {
      if (node->data == data)
	{
	  return node;
	}

      parent = node;
      if (data < node->data) 
	{
	  node = node->left;
	}
      else
	{
	  node = node->right;
	}
    }

  // No exact match could be found. 
  assert(!isNull(parent));
  switch (direction)
    {
    case exact:
      return NULL;
    case before:
      if (parent->data < data)
	{
	  return parent;
	}
      else
	{ 
	  return prev(parent);
	}
    case after:
      if (data < parent->data)
	{
	  return parent;
	}
      else
	{
	  return next(parent);
	}
    }

  // Can't get here, but the compiler is still complaining!
  assert(false);
  return NULL;
}  

template <class dataType>
rbtreeType<dataType>::iteratorType rbtreeType<dataType>::next(iteratorType node) const
{
  if (!node)
    { // Wrap around.
      return first();
    }

  if (!isNull(node->right))
    {
      node = node->right;
      while (!isNull(node->left))
	{
	  node = node->left;
	}
      return node;
    }

  nodeType *parent = node->parent;
  while ((!isNull(parent)) && (node == parent->right))
    {
      node = parent;
      parent = node->parent;
    }
  return export(parent);
}

template <class dataType>
rbtreeType<dataType>::iteratorType rbtreeType<dataType>::prev(iteratorType node) const
{
  if (!node)
    { // Wrap around.
      return last();
    }

  if (!isNull(node->left))
    {
      node = node->left;
      while (!isNull(node->right))
	{
	  node = node->right;
	}
      return node;
    }

  nodeType *parent = node->parent;
  while ((!isNull(parent)) && (node == parent->left))
    {
      node = parent;
      parent = node->parent;
    }
  return export(parent);
}



#ifdef UNIT_TEST
////////////////////////////////////////////////////////////////////////////
//  Unit test code.
////////////////////////////////////////////////////////////////////////////

typedef rbtreeType<int> testType;
typedef testType::nodeType *iteratorType;

testType tree;

ostream &operator << (ostream &s, testType::nodeType *n)
{
  if (n)
    {
      s<<n->getData();
    }
  else
    {
      s<<"NULL";
    }
  return s;
}

void doAdd(int value)
{
  cout<<"Add "<<value<<":  ";
  tree.add(value);
  tree.assertInvariants();
  tree.dump();
  cout<<'\n';
}

void doRemove(int value)
{
  cout<<"Remove "<<value<<":  ";
  tree.remove(value);
  tree.assertInvariants();
  tree.dump();
  cout<<'\n';
}

int main() 
{
  assert(cout<<"Assertions enabled.\n");

  tree.assertInvariants();

  cout<<"\nEmpty tree:\n";
  cout<<"tree.first() = "<<tree.first()<<'\n';
  cout<<"tree.last() = "<<tree.last()<<'\n';

  cout<<'\n';
  doAdd(5);
  doAdd(7);
  doAdd(9);
  doAdd(1);
  doAdd(8);
  doAdd(4);
  iteratorType i;
  cout<<"Forward:  ";
  for (i = tree.first(); i; i = tree.next(i))
    {
      cout<<i<<", ";
    }
  cout<<i<<'\n';
  cout<<"And back:  ";
  do
    {
      cout<<i<<", ";
      i = tree.prev(i);
    }
  while (i);
  cout<<i<<'\n';

  cout<<'\n';
  doAdd(1);
  doAdd(2);
  doAdd(3);
  doAdd(11);
  cout<<"Forward:  ";
  for (i = tree.first(); i; i = tree.next(i))
    {
      cout<<i<<", ";
    }
  cout<<i<<'\n';

  int j;
  for (j=0; j<13; j++)
    {
      cout<<j<<" is ";
      if (!tree.exists(j))
	{
	  cout<<"not ";
	}
      cout<<"present\n";
    }

  for (j=0; j<13; j++)
    {
      cout<<"find("<<j<<", lower) = "<<tree.find(j, testType::lower)
	  <<", find("<<j<<", higher) = "<<tree.find(j, textType::higher)
	  <<", find("<<j<<", exact) = "<<tree.find(j, textType::exact)<<'\n';
    }

  cout<<'\n';
  doRemove(3);
  doRemove(5);
  doRemove(6);
  doRemove(7);
  cout<<"Forward:  ";
  for (i = tree.first(); i; i = tree.next(i))
    {
      cout<<i<<", ";
    }
  cout<<i<<'\n';

  cout<<'\n';
  for (j = 0; j < 1000; j++)
    {
      cout<<'.';
      int k = rand()+j;
      if (j%2)
	{
	  tree.add(k);
	  assert(tree.exists(k));
	}
      else
	{
	  tree.remove(k);
	  assert(!tree.exists(k));
	}
      tree.assertInvariants();
    }
  cout<<'\n';
  tree.dump();

  return 0;
}
#endif

#endif
